/*
 * Copyright 2011-2013 Jiří Janoušek <janousek.jiri@gmail.com>
 * Copyright 2014 Stefano Bagnatica <thepisu@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Anonymous function is used not to pollute environment */
(function(Nuvola)
{	
	/**
	 * Creates integration binded to Nuvola JS API
	 */
	var Integration = function()
	{
		/* Overwrite default command function */
		Nuvola.onMessageReceived = Nuvola.bind(this, this.messageHandler);
		
		/* For debug output */
		this.name = "Jango";

		// Jango does not support going to previous
		Nuvola.updateAction(Nuvola.ACTION_PREV_SONG, false);
		
		/* Let's run */
		this.state = Nuvola.STATE_NONE;
		this.can_thumbs_up = null;
		this.can_thumbs_down = null;
		this.can_next = null;
		this.update();
	};
	
	/**
	 * Get DOM element by id, from the document of Jango content frame
	 * @param id Element id
	 */
	Integration.prototype.getJangoElement = function(id)
	{
		return top.content.document.getElementById(id);
	}
	
	/**
	 * Emulate click on an element in Jango content frame
	 * @param id Element id
	 */
	Integration.prototype.clickJangoButton = function(id)
	{
		Nuvola.clickOnElement(this.getJangoElement(id));
	}
	
	/**
	 * Find if Jango is playing, looking for button class name
	 */
	Integration.prototype.isPlaying = function()
	{
		var el = this.getJangoElement("btn-playpause");
		if (el == null) return false;
		return (el.className == "player_ctrls pause");
	}
	
	/**
	 * Click automatically the commit button after thumb up / down
	 * @param trycount Trying counter for button found
	 */
	Integration.prototype.autoCommit = function(trycount)
	{
		// find the commit button
		var els = top.content.document.getElementsByName("commit");
		if (els != null && els.length > 0) {
			var element = els[0];
			Nuvola.clickOnElement(element);
		} else {
			// the commit button coul not be ready... retry for max 4 times
			if (trycount==undefined) trycount=1;
			if (trycount<4) {
				var integrationObject = this;
				setTimeout(function() { integrationObject.autoCommit(trycount+1) }, 250);
			}
		}
	}
	
	/**
	 * Updates current playback state
	 */
	Integration.prototype.update = function()
	{
		// Default values
		var state = Nuvola.STATE_NONE;
		var can_next = false;
		var can_thumbs_up = false;
		var can_thumbs_down = false;
		var album_art = null;
		var song = null;
		var artist = null;
		var album = null;
		
		// top.content is null, you have not selected a station yet...
		if (top.content != null) {
			// player active, can do actions
			can_next = true;
			can_thumbs_up = true;
			can_thumbs_down = true;
			
			// Retrieve song details
			var el = this.getJangoElement('current-song');
			if (el != null) song = el.textContent.trim();
			
			if (song == null || song == "Loading Music…") {
				// is still loading music... wait some more...
				setTimeout(Nuvola.bind(this, this.update), 250);
				return;
			} else {
				// Retrieve playing status details
				if (this.isPlaying()) {
					state = Nuvola.STATE_PLAYING;			
				} else {
					state = Nuvola.STATE_PAUSED;
				}
				
				// Retrieve artist details
				el = this.getJangoElement('player_current_artist');
				if (el != null) {
					var elLink = el.getElementsByTagName("a");
					if (elLink.length > 0) artist = elLink[0].textContent.trim();
					else artist = el.textContent.trim();
				}
				
				// I use station name as album name (no album declared in Jango)
				el = this.getJangoElement("nowpln");
				if (el != null) album = el.title;
				
				// I use current song image as album art
				el = this.getJangoElement('player_main_pic_img');
				if (el != null) album_art = el.src;
			}
		}
		
		// Save state
		this.state = state;
		
		// Submit data to Nuvola backend
		Nuvola.updateSong(song, artist, album, album_art, state);
		
		// Update actions
		if (this.can_next !== can_next)
		{
			this.can_next = can_next;
			Nuvola.updateAction(Nuvola.ACTION_NEXT_SONG, can_next);
		}
		if (this.can_thumbs_up !== can_thumbs_up)
		{
			this.can_thumbs_up = can_thumbs_up;
			Nuvola.updateAction(Nuvola.ACTION_THUMBS_UP, can_thumbs_up);
		}
		if (this.can_thumbs_down !== can_thumbs_down)
		{
			this.can_thumbs_down = can_thumbs_down;
			Nuvola.updateAction(Nuvola.ACTION_THUMBS_DOWN, can_thumbs_down);
		}
		
		// Schedule update
		setTimeout(Nuvola.bind(this, this.update), 500);
	}
	
	/**
	 * Message handler
	 * @param cmd command to execute
	 */
	Integration.prototype.messageHandler = function(cmd)
	{
		/* Respond to user actions */
		try
		{
			switch (cmd)
			{
				case Nuvola.ACTION_PLAY:
					if (this.state != Nuvola.STATE_PLAYING)
						this.clickJangoButton("btn-playpause");
					break;
				case Nuvola.ACTION_PAUSE:
					this.clickJangoButton("btn-playpause");
					break;
				case Nuvola.ACTION_TOGGLE_PLAY:
					this.clickJangoButton("btn-playpause");
					break;
				case Nuvola.ACTION_NEXT_SONG:
					this.clickJangoButton("btn-ff");
					break;
				case Nuvola.ACTION_THUMBS_UP:
					this.clickJangoButton("btn-fav");
					setTimeout(this.autoCommit, 250);
					break;
				case Nuvola.ACTION_THUMBS_DOWN:
					this.clickJangoButton("player_ban");
					setTimeout(this.autoCommit, 250);
					break;
				default:
					// Other commands are not supported
					throw {"message": "Not supported."};
			}
			console.log(this.name + ": command '" + cmd + "' executed.");
		}
		catch (e)
		{
			// Older API expected exception to be a string.
			throw (this.name + ": " + e.message);
		}
	}
	
	/* Store reference */ 
	Nuvola.integration = new Integration(); // Singleton

// Immediately call the anonymous function with Nuvola JS API main object as an argument.
// Note that "this" is set to the Nuvola JS API main object.
})(this);
